/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2001 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: gdsl_hash.h
 * $Revision: 1.3 $
 * $Date: 2002/02/21 09:55:51 $
 * $Author: ndarnis $
 * $Id: gdsl_hash.h,v 1.3 2002/02/21 09:55:51 ndarnis Exp $
 */

#ifndef _GDSL_HASH_H_
#define _GDSL_HASH_H_

#include "gdsl_types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct hash_table *gdsl_hash_t;

/* 
 * Hash function type
 */
typedef ulong (*gdsl_hash_func_t)( ulong );

/*
 * Key function type
 */
typedef ulong (*gdsl_key_func_t)( gdsl_element_t );

/******************************************************************************/
/* Management functions                                                       */
/******************************************************************************/

/*
 * Creates a new hash-table.
 * Creates a new allocated hash-table data structure which name is set to a copy
 * of NAME unless GDSL_HASH_NAME is NULL. The hash-table can contain a maximum 
 * of HASH_ELT_NB elements.
 * In case of success, returns the newly allocated data structure.
 * In case of failure, returns NULL.
 * The two functions pointers ALLOC_F and FREE_F could be used to, 
 * respectively, alloc and free elements in the hash.
 *
 * Complexity: O( 1 )
 *
 * Requires: ALLOC_F & FREE_F must be defined as explained in gdsl_types.h
 *           & KEY_F & HASH_F must be defined as explained at top of this file
 *
 * NOTES:
 * The functions pointers could be set to NULL to use the default ones:
 * The default ALLOC_F simply returns its argument.
 * The default FREE_F does nothing.
 * The default KEY_F is described in gdsl_hash.c.
 * The default HASH_F is described in gdsl_hash.c.
 */
extern gdsl_hash_t
gdsl_hash_create (const char *NAME,               /* The name of the new hash */
		  gdsl_key_func_t KEY_F,                      /* Key function */
		  gdsl_alloc_func_t ALLOC_F,        /* Alloc function pointer */
		  gdsl_free_func_t FREE_F,           /* Free function pointer */
		  gdsl_hash_func_t HASH_F,                   /* Hash function */
		  ulong HASH_ELT_NB    /* Max number elements of the new hash */
		  );

/*
 * Destroy a hash-table.
 * Destroy all the elements of H by calling H's FREE_F function passed to 
 * gdsl_hash_create(). Destroy H itself too.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern void
gdsl_hash_destroy (gdsl_hash_t H                       /* The hash to destroy */
		   );

/*
 * Flush a hash-table.
 * Same as above but H is not destroyed (only the elements are removed and 
 * free-ed by calling L's FREE_F function passed to gdsl_hash_create()).
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern void
gdsl_hash_flush (gdsl_hash_t H                           /* The hash to flush */
		 );
  
/*
 * Sets the name of a hash-table.
 * Deallocate the previous H's name if exists, and change it to a copy of
 * GDSL_HASH_NEW_NAME. 
 * In case of success, returns the modified data structure.
 * In case of failure, returns NULL.
 *
 * Complexity: O( 1 )
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern gdsl_hash_t
gdsl_hash_set_name (gdsl_hash_t H,       /* The hash the name must be changed */
		    const char *NEW_NAME                     /* New name of H */
		    );

/******************************************************************************/
/* Information functions                                                      */
/******************************************************************************/

/*
 * Gets the name of a hash-table.
 * Returns H's name (and not a copy of it).
 *
 * Complexity: O( 1 )
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern const char *
gdsl_hash_get_name (gdsl_hash_t H            /* The hash to get the name from */
		    );
  
/*
 * Gets the fill factor of a hash-table.
 * Computes and returns the fill factor of H.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern double
gdsl_hash_fill_factor (gdsl_hash_t H   /* The hash to compute the factor from */
		       );

/******************************************************************************/
/* Insert functions                                                           */
/******************************************************************************/

/*
 * Inserts an element into a hash-table.
 * Allocates a new element using E and ALLOC_F passed to gdsl_hash_create() and 
 * insert it into H. The new element's key is computed using KEY_F passed to
 * gdsl_hash_create().
 * In case of success, returns the newly allocated element. 
 * In case of failure, returns NULL.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern gdsl_element_t
gdsl_hash_insert (gdsl_hash_t H,                     /* The hash to insert in */
		  void *E          /* The info used to create the new element */
		  );
  
/******************************************************************************/
/* Remove functions                                                           */
/******************************************************************************/

/*
 * Removes an element from a hash-table.
 * Removes from H the first founded element using KEY_F( E ) to find it. If the
 * element is found, it is returned. If no element is found, NULL is returned.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern gdsl_element_t
gdsl_hash_remove (gdsl_hash_t H,                   /* The hash to remove from */
		  void *E      /* The info used to get the key used to search */
		  );
  
/******************************************************************************/
/* Search functions                                                           */
/******************************************************************************/

/*
 * Search for an element into a hash-table.
 * Search into H for an element using KEY_F( E ) to compute its position. 
 * KEY_F is the function pointer passed to gdsl_hash_create().
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create()
 */
extern gdsl_element_t
gdsl_hash_search (gdsl_hash_t H,                     /* The hash to search in */
		  void *E      /* The info used to get the key used to search */
		  );

/******************************************************************************/
/* Parse functions                                                            */
/******************************************************************************/

/*
 * Parse all elements of a hash-table.
 * Maps all H's elements and launch MAP_F on each of them, with USER_DATA 
 * argument. If MAP_F returns 0 the gdsl_hash_map() function stops, otherwise it 
 * continues until end of H.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create() & MAP_F != NULL
 */
extern void
gdsl_hash_map (gdsl_hash_t H,                            /* The hash to parse */
	       gdsl_map_func_t MAP_F,                   /* The parse function */
	       void *USER_DATA                         /* Additional argument */
	       );

/******************************************************************************/
/* Output functions                                                           */
/******************************************************************************/

/*
 * Writes all elements of a hash-table to a file.
 * Writes H's elements to OUTPUT_FILE using WRITE_F function.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create() & OUTPUT_FILE != NULL
 *           & WRITE_F != NULL
 */
extern void
gdsl_hash_write (gdsl_hash_t H,                          /* The hash to write */
		 gdsl_write_func_t WRITE_F, /* Func. used to write H elements */
		 FILE *OUTPUT_FILE,   /* The file where to write H's elements */
		 void *USER_DATA       /* User informations passed to WRITE_F */
		 );

/*
 * Writes all elements of a hash-table to a file into XML language.
 * Writes H's elements to OUTPUT_FILE, into XML language.
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 *
 * Complexity:
 *
 * Requires: Q successfully allocated by gdsl_queue_create() 
 *           & OUTPUT_FILE != NULL
 */
extern void
gdsl_hash_write_xml (gdsl_hash_t H,                      /* The hash to write */
		     gdsl_write_func_t WRITE_F,         /* The write function */
		     FILE *OUTPUT_FILE,             /* The file to write into */
		     void *USER_DATA   /* User informations passed to WRITE_F */
		     );

/*
 * Dumps the whole hash-table to a file.
 * Dumps H to OUTPUT_FILE.
 * If WRITE_F != NULL, then use WRITE_F to write elements to OUTPUT_FILE.
 * Additionnal USER_DATA informations may be passed to WRITE_F.
 *
 * Complexity:
 *
 * Requires: H successfully allocated by gdsl_hash_create() & OUTPUT_FILE != NULL
 */
extern void
gdsl_hash_dump (gdsl_hash_t H,                            /* The hash to dump */
		gdsl_write_func_t WRITE_F,  /* Func. used to write H elements */
		FILE *OUTPUT_FILE,    /* The file where to write H's elements */
		void *USER_DATA        /* User informations passed to WRITE_F */
		);

#ifdef __cplusplus
}
#endif

#endif /* _GDSL_HASH_H_ */
